h1, h2, .rst-content .toctree-wrapper p.caption, h3, h4, h5, h6, legend {
    font-family: inherit;
}

.wy-nav-content a:visited {
    color: #3091d1;
}

.wy-side-nav-search {
    background-color: initial;
}

.wy-nav-side, .wy-nav-top {
    background-color: #272525;
}

.rst-content tt.literal, .rst-content tt.literal, .rst-content code.literal {
    color: #272525;
}

.wy-side-nav-search input[type="text"] {
    border-color: transparent;
}

.wy-nav-content {
    margin: initial;
}

.rst-content div[role=navigation], footer {
   font-size: 0.85em;
   color: #999;
}

.rst-content div[role=navigation] hr {
    margin-top: 6px;
}

footer hr {
    margin-bottom: 6px;
}

.rst-footer-buttons {
    display: none;
}

table td, table th {
    white-space: normal !important;
    line-height: 20px;
}
